@@ -24,7 +24,6 @@ class RecipientInterface(CBORProcessor):
         key_ops: List[int] = [],
         key: bytes = b"",
     ):
-
         """
         Constructor.
 
@@ -137,7 +136,7 @@ class RecipientInterface(CBORProcessor):
         return self._ciphertext
 
     @property
-    def recipients(self) -> Union[List[Any], None]:
+    def recipients(self) -> List[Any]:
         """
         The list of recipient information structures.
         """
