@@ -1,6 +1,7 @@
 import json
 import os
 import zlib
+from typing import Any, Dict, Union
 
 import requests
 from base45 import b45decode
@@ -13,8 +14,8 @@ class Verifier:
     def __init__(self, base_url: str, trustlist_store_path: str):
         self._base_url = base_url
         self._trustlist_store_path = trustlist_store_path
-        self._dscs = []
-        self._trustlist = []
+        self._dscs: list = []
+        self._trustlist: list = []
         self._load_trustlist()
 
     @classmethod
@@ -63,7 +64,7 @@ class Verifier:
             json.dump([v for v in self._trustlist if v["x_kid"] in active_kids], f, indent=4)
         return
 
-    def verify_and_decode(self, eudcc: bytes) -> bytes:
+    def verify_and_decode(self, eudcc: bytes) -> Union[Dict[int, Any], bytes]:
         if eudcc.startswith(b"HC1:"):
             # Decode Base45 data.
             eudcc = b45decode(eudcc[4:])
