@@ -2,10 +2,10 @@ from secrets import token_bytes
 
 import pytest
 
-from cwt import COSE, COSEKey, Recipient, Signer
+from cwt import COSE, COSEKey, COSEMessage, Recipient, Signer
 
 
-class TestCOSESample:
+class TestCOSESampleWithEncode:
     """
     Tests for samples on COSE Usage Examples.
     """
@@ -41,6 +41,46 @@ class TestCOSESample:
 
         assert encoded == encoded2 == encoded3
 
+    def test_cose_usage_examples_cose_mac0_countersignature(self):
+        mac_key = COSEKey.generate_symmetric_key(alg="HS256", kid="01")
+
+        # The sender side:
+        sender = COSE.new(alg_auto_inclusion=True, kid_auto_inclusion=True)
+        encoded = sender.encode(b"Hello world!", mac_key)
+
+        # The notary side:
+        notary = Signer.from_jwk(
+            {
+                "kid": "01",
+                "kty": "OKP",
+                "crv": "Ed25519",
+                "alg": "EdDSA",
+                "x": "2E6dX83gqD_D0eAmqnaHe1TC1xuld6iAKXfw2OVATr0",
+                "d": "L8JS08VsFZoZxGa9JvzYmCWOwg7zaKcei3KZmYsj7dc",
+            },
+        )
+        countersigned = COSEMessage.loads(encoded).countersign(notary).dumps()
+
+        # The recipient side:
+        pub_key = COSEKey.from_jwk(
+            {
+                "kid": "01",
+                "kty": "OKP",
+                "crv": "Ed25519",
+                "alg": "EdDSA",
+                "x": "2E6dX83gqD_D0eAmqnaHe1TC1xuld6iAKXfw2OVATr0",
+            },
+        )
+        recipient = COSE.new()
+        assert b"Hello world!" == recipient.decode(countersigned, mac_key)
+        try:
+            sig = COSEMessage.loads(countersigned).counterverify(pub_key)
+        except Exception as err:
+            pytest.fail(f"failed to verify: {err}")
+        countersignature = COSEMessage.from_cose_signature(sig)
+        assert countersignature.protected[1] == -8  # alg: "EdDSA"
+        assert countersignature.unprotected[4] == b"01"  # kid: b"01"
+
     def test_cose_usage_examples_cose_mac_direct(self):
         mac_key = COSEKey.generate_symmetric_key(alg="HS512", kid="01")
 
@@ -66,7 +106,6 @@ class TestCOSESample:
         assert encoded == encoded2 == encoded3
 
     def test_cose_usage_examples_cose_mac_direct_hkdf_sha_256(self):
-
         shared_material = token_bytes(32)
         shared_key = COSEKey.from_symmetric_key(shared_material, kid="01")
 
@@ -90,7 +129,6 @@ class TestCOSESample:
         assert b"Hello world!" == recipient.decode(encoded, shared_key, context={"alg": "HS256"})
 
     def test_cose_usage_examples_cose_mac_aes_key_wrap(self):
-
         # The sender side:
         mac_key = COSEKey.generate_symmetric_key(alg="HS512")
         enc_key = COSEKey.from_jwk(
@@ -110,7 +148,6 @@ class TestCOSESample:
         assert b"Hello world!" == recipient.decode(encoded, enc_key)
 
     def test_cose_usage_examples_cose_mac_ecdh_direct_hkdf_p256(self):
-
         # The sender side:
         # The following key is provided by the recipient in advance.
         pub_key = COSEKey.from_jwk(
@@ -154,7 +191,6 @@ class TestCOSESample:
         assert b"Hello world!" == recipient.decode(encoded, priv_key, context={"alg": "HS256"})
 
     def test_cose_usage_examples_cose_mac_ecdh_ss_a128kw(self):
-
         # The sender side:
         mac_key = COSEKey.generate_symmetric_key(alg="HS256")
         priv_key = COSEKey.from_jwk(
@@ -205,7 +241,6 @@ class TestCOSESample:
         assert b"Hello world!" == recipient.decode(encoded, priv_key, context={"alg": "HS256"})
 
     def test_cose_usage_examples_cose_mac_ecdh_aes_key_wrap(self):
-
         # The sender side:
         mac_key = COSEKey.generate_symmetric_key(alg="HS256")
         pub_key = COSEKey.from_jwk(
@@ -245,6 +280,100 @@ class TestCOSESample:
         )
         assert b"Hello world!" == recipient.decode(encoded, priv_key, context={"alg": "HS256"})
 
+    def test_cose_usage_examples_cose_mac_countersignature(self):
+        mac_key = COSEKey.generate_symmetric_key(alg="HS256", kid="01")
+
+        # The sender side:
+        r = Recipient.new(unprotected={"alg": "direct", "kid": mac_key.kid})
+        sender = COSE.new()
+        encoded = sender.encode(b"Hello world!", mac_key, protected={"alg": "HS256"}, recipients=[r])
+
+        # The notary side:
+        notary = Signer.from_jwk(
+            {
+                "kid": "01",
+                "kty": "OKP",
+                "crv": "Ed25519",
+                "alg": "EdDSA",
+                "x": "2E6dX83gqD_D0eAmqnaHe1TC1xuld6iAKXfw2OVATr0",
+                "d": "L8JS08VsFZoZxGa9JvzYmCWOwg7zaKcei3KZmYsj7dc",
+            },
+        )
+        countersigned = COSEMessage.loads(encoded).countersign(notary).dumps()
+
+        # print(countersigned.hex())
+
+        # The recipient side:
+        pub_key = COSEKey.from_jwk(
+            {
+                "kid": "01",
+                "kty": "OKP",
+                "crv": "Ed25519",
+                "alg": "EdDSA",
+                "x": "2E6dX83gqD_D0eAmqnaHe1TC1xuld6iAKXfw2OVATr0",
+            },
+        )
+        recipient = COSE.new()
+        assert b"Hello world!" == recipient.decode(countersigned, mac_key)
+        try:
+            sig = COSEMessage.loads(countersigned).counterverify(pub_key)
+        except Exception as err:
+            pytest.fail(f"failed to verify: {err}")
+        countersignature = COSEMessage.from_cose_signature(sig)
+        assert countersignature.protected[1] == -8  # alg: "EdDSA"
+        assert countersignature.unprotected[4] == b"01"  # kid: b"01"
+
+    def test_cose_usage_examples_cose_mac_hpke(self):
+        mac_key = COSEKey.generate_symmetric_key(alg="HS256")
+
+        # The sender side:
+        rpk = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "kid": "01",
+                "crv": "P-256",
+                "x": "usWxHK2PmfnHKwXPS54m0kTcGJ90UiglWiGahtagnv8",
+                "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
+            }
+        )
+        r = Recipient.new(
+            protected={
+                1: -1,  # alg: "HPKE"
+            },
+            unprotected={
+                4: b"01",  # kid: "01"
+                -4: [  # HPKE sender information
+                    0x0010,  # kem: DHKEM(P-256, HKDF-SHA256)
+                    0x0001,  # kdf: HKDF-SHA256
+                    0x0001,  # aead: AES-128-GCM
+                ],
+            },
+            recipient_key=rpk,
+        )
+        sender = COSE.new()
+        encoded = sender.encode(
+            b"This is the content.",
+            mac_key,
+            protected={1: 5},  # alg: HS256
+            recipients=[r],
+        )
+
+        # print(encoded.hex())
+
+        # The recipient side:
+        rsk = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "kid": "01",
+                "crv": "P-256",
+                "x": "usWxHK2PmfnHKwXPS54m0kTcGJ90UiglWiGahtagnv8",
+                "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
+                "d": "V8kgd2ZBRuh2dgyVINBUqpPDr7BOMGcF22CQMIUHtNM",
+            }
+        )
+        recipient = COSE.new()
+        assert b"This is the content." == recipient.decode(encoded, rsk)
+
     def test_cose_usage_examples_cose_encrypt0(self):
         enc_key = COSEKey.generate_symmetric_key(alg="ChaCha20/Poly1305", kid="01")
 
@@ -277,6 +406,47 @@ class TestCOSESample:
 
         # assert encoded == encoded2 == encoded3
 
+    def test_cose_usage_examples_cose_encrypt0_countersignature(self):
+        enc_key = COSEKey.generate_symmetric_key(alg="ChaCha20/Poly1305", kid="01")
+
+        # The sender side:
+        nonce = enc_key.generate_nonce()
+        sender = COSE.new(alg_auto_inclusion=True, kid_auto_inclusion=True)
+        encoded = sender.encode(b"Hello world!", enc_key, unprotected={5: nonce})
+
+        # The notary side:
+        notary = Signer.from_jwk(
+            {
+                "kid": "01",
+                "kty": "OKP",
+                "crv": "Ed25519",
+                "alg": "EdDSA",
+                "x": "2E6dX83gqD_D0eAmqnaHe1TC1xuld6iAKXfw2OVATr0",
+                "d": "L8JS08VsFZoZxGa9JvzYmCWOwg7zaKcei3KZmYsj7dc",
+            },
+        )
+        countersigned = COSEMessage.loads(encoded).countersign(notary).dumps()
+
+        # The recipient side:
+        pub_key = COSEKey.from_jwk(
+            {
+                "kid": "01",
+                "kty": "OKP",
+                "crv": "Ed25519",
+                "alg": "EdDSA",
+                "x": "2E6dX83gqD_D0eAmqnaHe1TC1xuld6iAKXfw2OVATr0",
+            },
+        )
+        recipient = COSE.new()
+        assert b"Hello world!" == recipient.decode(countersigned, enc_key)
+        try:
+            sig = COSEMessage.loads(countersigned).counterverify(pub_key)
+        except Exception as err:
+            pytest.fail(f"failed to verify: {err}")
+        countersignature = COSEMessage.from_cose_signature(sig)
+        assert countersignature.protected[1] == -8  # alg: "EdDSA"
+        assert countersignature.unprotected[4] == b"01"  # kid: b"01"
+
     def test_cose_usage_examples_cose_encrypt0_hpke(self):
         # The sender side:
         rpk = COSEKey.from_jwk(
@@ -299,11 +469,11 @@ class TestCOSESample:
             },
             unprotected={
                 4: b"01",  # kid: "01"
-                -4: {  # HPKE sender information
-                    1: 0x0010,  # kem: DHKEM(P-256, HKDF-SHA256)
-                    2: 0x0001,  # kdf: HKDF-SHA256
-                    3: 0x0001,  # aead: AES-128-GCM
-                },
+                -4: [  # HPKE sender information
+                    0x0010,  # kem: DHKEM(P-256, HKDF-SHA256)
+                    0x0001,  # kdf: HKDF-SHA256
+                    0x0001,  # aead: AES-128-GCM
+                ],
             },
         )
 
@@ -357,8 +527,57 @@ class TestCOSESample:
 
         assert encoded == encoded2
 
-    def test_cose_usage_examples_cose_encrypt_hpke(self):
+    def test_cose_usage_examples_cose_encrypt_countersignature(self):
+        enc_key = COSEKey.generate_symmetric_key(alg="ChaCha20/Poly1305", kid="01")
 
+        # The sender side:
+        nonce = enc_key.generate_nonce()
+        r = Recipient.new(unprotected={"alg": "direct"})
+
+        sender = COSE.new()
+        encoded = sender.encode(
+            b"Hello world!",
+            enc_key,
+            protected={"alg": "ChaCha20/Poly1305"},
+            unprotected={"kid": enc_key.kid, "iv": nonce},
+            recipients=[r],
+        )
+
+        # The notary side:
+        notary = Signer.from_jwk(
+            {
+                "kid": "01",
+                "kty": "OKP",
+                "crv": "Ed25519",
+                "alg": "EdDSA",
+                "x": "2E6dX83gqD_D0eAmqnaHe1TC1xuld6iAKXfw2OVATr0",
+                "d": "L8JS08VsFZoZxGa9JvzYmCWOwg7zaKcei3KZmYsj7dc",
+            },
+        )
+        countersigned = COSEMessage.loads(encoded).countersign(notary).dumps()
+
+        # The recipient side:
+        pub_key = COSEKey.from_jwk(
+            {
+                "kid": "01",
+                "kty": "OKP",
+                "crv": "Ed25519",
+                "alg": "EdDSA",
+                "x": "2E6dX83gqD_D0eAmqnaHe1TC1xuld6iAKXfw2OVATr0",
+            },
+        )
+        recipient = COSE.new()
+        assert b"Hello world!" == recipient.decode(countersigned, enc_key)
+
+        try:
+            sig = COSEMessage.loads(countersigned).counterverify(pub_key)
+        except Exception as err:
+            pytest.fail(f"failed to verify: {err}")
+        countersignature = COSEMessage.from_cose_signature(sig)
+        assert countersignature.protected[1] == -8  # alg: "EdDSA"
+        assert countersignature.unprotected[4] == b"01"  # kid: b"01"
+
+    def test_cose_usage_examples_cose_encrypt_hpke(self):
         # The sender side:
         enc_key = COSEKey.generate_symmetric_key(alg="A128GCM")
         rpk = COSEKey.from_jwk(
@@ -376,11 +595,11 @@ class TestCOSESample:
             },
             unprotected={
                 4: b"01",  # kid: "01"
-                -4: {  # HPKE sender information
-                    1: 0x0010,  # kem: DHKEM(P-256, HKDF-SHA256)
-                    2: 0x0001,  # kdf: HKDF-SHA256
-                    3: 0x0001,  # aead: AES-128-GCM
-                },
+                -4: [  # HPKE sender information
+                    0x0010,  # kem: DHKEM(P-256, HKDF-SHA256)
+                    0x0001,  # kdf: HKDF-SHA256
+                    0x0001,  # aead: AES-128-GCM
+                ],
             },
             recipient_key=rpk,
         )
@@ -411,7 +630,6 @@ class TestCOSESample:
         assert b"This is the content." == recipient.decode(encoded, rsk)
 
     def test_cose_usage_examples_cose_encrypt_hpke_with_1st_layer_hpke(self):
-
         # The sender side:
         # enc_key = COSEKey.generate_symmetric_key(alg="A128GCM")
         rpk = COSEKey.from_jwk(
@@ -429,11 +647,11 @@ class TestCOSESample:
             },
             unprotected={
                 4: b"01",  # kid: "01"
-                -4: {  # HPKE sender information
-                    1: 0x0010,  # kem: DHKEM(P-256, HKDF-SHA256)
-                    2: 0x0001,  # kdf: HKDF-SHA256
-                    3: 0x0001,  # aead: AES-128-GCM
-                },
+                -4: [  # HPKE sender information
+                    0x0010,  # kem: DHKEM(P-256, HKDF-SHA256)
+                    0x0001,  # kdf: HKDF-SHA256
+                    0x0001,  # aead: AES-128-GCM
+                ],
             },
             recipient_key=rpk,
         )
@@ -446,11 +664,11 @@ class TestCOSESample:
                 },
                 unprotected={
                     4: b"xx",  # kid: "xx"
-                    -4: {  # HPKE sender information
-                        1: 0x0010,  # kem: DHKEM(P-256, HKDF-SHA256)
-                        2: 0x0001,  # kdf: HKDF-SHA256
-                        3: 0x0001,  # aead: AES-128-GCM
-                    },
+                    -4: [  # HPKE sender information
+                        0x0010,  # kem: DHKEM(P-256, HKDF-SHA256)
+                        0x0001,  # kdf: HKDF-SHA256
+                        0x0001,  # aead: AES-128-GCM
+                    ],
                 },
                 recipients=[r],
             )
@@ -458,7 +676,6 @@ class TestCOSESample:
         assert "key should be set." in str(err.value)
 
     def test_cose_usage_examples_cose_encrypt_hpke_with_nonce(self):
-
         # The sender side:
         # enc_key = COSEKey.generate_symmetric_key(alg="A128GCM")
         rpk = COSEKey.from_jwk(
@@ -476,11 +693,11 @@ class TestCOSESample:
             },
             unprotected={
                 4: b"01",  # kid: "01"
-                -4: {  # HPKE sender information
-                    1: 0x0010,  # kem: DHKEM(P-256, HKDF-SHA256)
-                    2: 0x0001,  # kdf: HKDF-SHA256
-                    3: 0x0001,  # aead: AES-128-GCM
-                },
+                -4: [  # HPKE sender information
+                    0x0010,  # kem: DHKEM(P-256, HKDF-SHA256)
+                    0x0001,  # kdf: HKDF-SHA256
+                    0x0001,  # aead: AES-128-GCM
+                ],
             },
             recipient_key=rpk,
         )
@@ -505,7 +722,6 @@ class TestCOSESample:
         assert "key should be set." in str(err.value)
 
     def test_cose_usage_examples_cose_encrypt_direct_hkdf_sha_256(self):
-
         shared_material = token_bytes(32)
         shared_key = COSEKey.from_symmetric_key(shared_material, kid="01")
 
@@ -553,7 +769,6 @@ class TestCOSESample:
         assert b"Hello world!" == recipient.decode(encoded, wrapping_key)
 
     def test_cose_usage_examples_cose_encrypt_ecdh_direct_hkdf_p256(self):
-
         # The sender side:
         pub_key = COSEKey.from_jwk(
             {
@@ -591,7 +806,6 @@ class TestCOSESample:
         assert b"Hello world!" == recipient.decode(encoded, priv_key, context={"alg": "A128GCM"})
 
     def test_cose_usage_examples_cose_encrypt_ecdh_ss_a128kw(self):
-
         # The sender side:
         enc_key = COSEKey.generate_symmetric_key(alg="A128GCM")
         nonce = enc_key.generate_nonce()
@@ -643,8 +857,48 @@ class TestCOSESample:
         )
         assert b"Hello world!" == recipient.decode(encoded, r_priv_key, context={"alg": "A128GCM"})
 
-    def test_cose_usage_examples_cose_signature1(self):
+    def test_cose_usage_examples_cose_encrypt_ecdh_aes_key_wrap(self):
+        enc_key = COSEKey.generate_symmetric_key(alg="A128GCM")
 
+        # The sender side:
+        pub_key = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "alg": "ECDH-ES+A128KW",
+                "kid": "01",
+                "crv": "P-256",
+                "x": "Ze2loSV3wrroKUN_4zhwGhCqo3Xhu1td4QjeQ5wIVR0",
+                "y": "HlLtdXARY_f55A3fnzQbPcm6hgr34Mp8p-nuzQCE0Zw",
+            }
+        )
+        r = Recipient.new(
+            unprotected={"alg": "ECDH-ES+A128KW"},
+            recipient_key=pub_key,
+            context={"alg": "A128GCM"},
+        )
+        sender = COSE.new(alg_auto_inclusion=True)
+        encoded = sender.encode(
+            b"Hello world!",
+            key=enc_key,
+            recipients=[r],
+        )
+
+        # The recipient side:
+        recipient = COSE.new()
+        priv_key = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "alg": "ECDH-ES+A128KW",
+                "kid": "01",
+                "crv": "P-256",
+                "x": "Ze2loSV3wrroKUN_4zhwGhCqo3Xhu1td4QjeQ5wIVR0",
+                "y": "HlLtdXARY_f55A3fnzQbPcm6hgr34Mp8p-nuzQCE0Zw",
+                "d": "r_kHyZ-a06rmxM3yESK84r1otSg-aQcVStkRhA-iCM8",
+            }
+        )
+        assert b"Hello world!" == recipient.decode(encoded, priv_key, context={"alg": "A128GCM"})
+
+    def test_cose_usage_examples_cose_signature1(self):
         # The sender side:
         priv_key = COSEKey.from_jwk(
             {
@@ -690,50 +944,64 @@ class TestCOSESample:
         )
         assert b"Hello world!" == recipient.decode(encoded3, pub_key)
 
-    def test_cose_usage_examples_cose_encrypt_ecdh_aes_key_wrap(self):
-
-        enc_key = COSEKey.generate_symmetric_key(alg="A128GCM")
-
+    def test_cose_usage_examples_cose_signature1_countersignature(self):
         # The sender side:
-        pub_key = COSEKey.from_jwk(
-            {
-                "kty": "EC",
-                "alg": "ECDH-ES+A128KW",
-                "kid": "01",
-                "crv": "P-256",
-                "x": "Ze2loSV3wrroKUN_4zhwGhCqo3Xhu1td4QjeQ5wIVR0",
-                "y": "HlLtdXARY_f55A3fnzQbPcm6hgr34Mp8p-nuzQCE0Zw",
-            }
-        )
-        r = Recipient.new(
-            unprotected={"alg": "ECDH-ES+A128KW"},
-            recipient_key=pub_key,
-            context={"alg": "A128GCM"},
-        )
-        sender = COSE.new(alg_auto_inclusion=True)
-        encoded = sender.encode(
-            b"Hello world!",
-            key=enc_key,
-            recipients=[r],
-        )
-
-        # The recipient side:
-        recipient = COSE.new()
         priv_key = COSEKey.from_jwk(
             {
                 "kty": "EC",
-                "alg": "ECDH-ES+A128KW",
                 "kid": "01",
                 "crv": "P-256",
-                "x": "Ze2loSV3wrroKUN_4zhwGhCqo3Xhu1td4QjeQ5wIVR0",
-                "y": "HlLtdXARY_f55A3fnzQbPcm6hgr34Mp8p-nuzQCE0Zw",
-                "d": "r_kHyZ-a06rmxM3yESK84r1otSg-aQcVStkRhA-iCM8",
+                "x": "usWxHK2PmfnHKwXPS54m0kTcGJ90UiglWiGahtagnv8",
+                "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
+                "d": "V8kgd2ZBRuh2dgyVINBUqpPDr7BOMGcF22CQMIUHtNM",
             }
         )
-        assert b"Hello world!" == recipient.decode(encoded, priv_key, context={"alg": "A128GCM"})
+        sender = COSE.new(alg_auto_inclusion=True, kid_auto_inclusion=True)
+        encoded = sender.encode(b"Hello world!", priv_key)
+
+        # The notary side:
+        notary = Signer.from_jwk(
+            {
+                "kid": "01",
+                "kty": "OKP",
+                "crv": "Ed25519",
+                "alg": "EdDSA",
+                "x": "2E6dX83gqD_D0eAmqnaHe1TC1xuld6iAKXfw2OVATr0",
+                "d": "L8JS08VsFZoZxGa9JvzYmCWOwg7zaKcei3KZmYsj7dc",
+            },
+        )
+        countersigned = COSEMessage.loads(encoded).countersign(notary).dumps()
+
+        # The recipient side:
+        pub_key = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "kid": "01",
+                "crv": "P-256",
+                "x": "usWxHK2PmfnHKwXPS54m0kTcGJ90UiglWiGahtagnv8",
+                "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
+            }
+        )
+        notary_pub_key = COSEKey.from_jwk(
+            {
+                "kid": "01",
+                "kty": "OKP",
+                "crv": "Ed25519",
+                "alg": "EdDSA",
+                "x": "2E6dX83gqD_D0eAmqnaHe1TC1xuld6iAKXfw2OVATr0",
+            },
+        )
+        recipient = COSE.new()
+        assert b"Hello world!" == recipient.decode(countersigned, pub_key)
+        try:
+            sig = COSEMessage.loads(countersigned).counterverify(notary_pub_key)
+        except Exception as err:
+            pytest.fail(f"failed to verify: {err}")
+        countersignature = COSEMessage.from_cose_signature(sig)
+        assert countersignature.protected[1] == -8  # alg: "EdDSA"
+        assert countersignature.unprotected[4] == b"01"  # kid: b"01"
 
     def test_cose_usage_examples_cose_signature(self):
-
         # The sender side:
         signer = Signer.from_jwk(
             {
@@ -761,37 +1029,60 @@ class TestCOSESample:
         )
         assert b"Hello world!" == recipient.decode(encoded, pub_key)
 
-        # variation
-        signer = Signer.new(
-            cose_key=COSEKey.from_jwk(
-                {
-                    "kty": "EC",
-                    "kid": "01",
-                    "crv": "P-256",
-                    "x": "usWxHK2PmfnHKwXPS54m0kTcGJ90UiglWiGahtagnv8",
-                    "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
-                    "d": "V8kgd2ZBRuh2dgyVINBUqpPDr7BOMGcF22CQMIUHtNM",
-                }
-            ),
-            protected={"alg": "ES256"},
-            unprotected={"kid": "01"},
+    def test_cose_usage_examples_cose_signature_countersignature(self):
+        # The sender side:
+        signer = Signer.from_jwk(
+            {
+                "kty": "EC",
+                "kid": "01",
+                "crv": "P-256",
+                "x": "usWxHK2PmfnHKwXPS54m0kTcGJ90UiglWiGahtagnv8",
+                "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
+                "d": "V8kgd2ZBRuh2dgyVINBUqpPDr7BOMGcF22CQMIUHtNM",
+            },
         )
-        encoded2 = sender.encode(b"Hello world!", signers=[signer])
-        assert b"Hello world!" == recipient.decode(encoded2, pub_key)
+        sender = COSE.new()
+        encoded = sender.encode(b"Hello world!", signers=[signer])
 
-        signer = Signer.new(
-            cose_key=COSEKey.from_jwk(
-                {
-                    "kty": "EC",
-                    "kid": "01",
-                    "crv": "P-256",
-                    "x": "usWxHK2PmfnHKwXPS54m0kTcGJ90UiglWiGahtagnv8",
-                    "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
-                    "d": "V8kgd2ZBRuh2dgyVINBUqpPDr7BOMGcF22CQMIUHtNM",
-                }
-            ),
-            protected={1: -7},
-            unprotected={4: b"01"},
+        # The notary side:
+        notary = Signer.from_jwk(
+            {
+                "kid": "01",
+                "kty": "OKP",
+                "crv": "Ed25519",
+                "alg": "EdDSA",
+                "x": "2E6dX83gqD_D0eAmqnaHe1TC1xuld6iAKXfw2OVATr0",
+                "d": "L8JS08VsFZoZxGa9JvzYmCWOwg7zaKcei3KZmYsj7dc",
+            },
         )
-        encoded3 = sender.encode(b"Hello world!", signers=[signer])
-        assert b"Hello world!" == recipient.decode(encoded3, pub_key)
+        countersigned = COSEMessage.loads(encoded).countersign(notary).dumps()
+
+        # The recipient side:
+        pub_key = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "kid": "01",
+                "crv": "P-256",
+                "x": "usWxHK2PmfnHKwXPS54m0kTcGJ90UiglWiGahtagnv8",
+                "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
+            }
+        )
+        notary_pub_key = COSEKey.from_jwk(
+            {
+                "kid": "01",
+                "kty": "OKP",
+                "crv": "Ed25519",
+                "alg": "EdDSA",
+                "x": "2E6dX83gqD_D0eAmqnaHe1TC1xuld6iAKXfw2OVATr0",
+            },
+        )
+        recipient = COSE.new()
+        assert b"Hello world!" == recipient.decode(encoded, pub_key)
+
+        try:
+            sig = COSEMessage.loads(countersigned).counterverify(notary_pub_key)
+        except Exception as err:
+            pytest.fail(f"failed to verify: {err}")
+        countersignature = COSEMessage.from_cose_signature(sig)
+        assert countersignature.protected[1] == -8  # alg: "EdDSA"
+        assert countersignature.unprotected[4] == b"01"  # kid: b"01"
